package org.system.newEditor;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import org.sprite.animed.Pnj;
import org.sprite.animed.Transport;
import org.system.Carte;

/**
 * Interface graphique qui affiche la carte dite et ou on dfinit les interactions avec l'utilisateur
 * @author Groupe 6 - 2009
 */
public class AfficheurCarte extends JPanel implements KeyListener, MouseListener, MouseMotionListener{
	
	private static final long serialVersionUID = 1L;
	
	public static final int MODE_AUCUN = 0;
	public static final int MODE_TEXTURE = 1;
	public static final int MODE_PNJ = 2;
	public static final int MODE_FRANCHISSABILITE = 3;
	public static final int MODE_ELEMENT = 4;
	public static final int MODE_TRANSPORT = 5;
	
	/** Conteneur de l'interface */
	private FenetreEditeur fenetre;
	/** Liste de cases sauves avant un changement */
	private static ListeSauvegardeCase listeAnnuler;
	/** Liste de cases sauves aprs avoir annul */
	private static ListeSauvegardeCase listeRetablir;
	/** Permet la slection et le dplacement/copie de plusieurs cases */
	private SelectionneurMultiple selectionneur;
	
	private ActionDeplacerPnj deplacementPnj;
	/** Boolen qui indique si on affiche la grille ou non */
	private boolean afficherGrille;
	/** Boolen qui indique si on affiche le curseur ou non */
	private boolean afficherCurseur;
	/** Mode d'dition */
	private int mode;
	/** Largeur de l'interface en pixels */
	private int largeur;
	/** Hauteur de l'interface en pixels */
	private int hauteur;
	/** Abcisse de l'interface par rapport a son conteneur*/
	private int x;
	/** Ordonne de l'interface par rapport a son conteneur*/
	private int y;
	/** Indice d'abcisse de la dernire case survole */
	private int caseX;
	/** Indice d'ordonne de la dernire case survole */
	private int caseY;
	/** Image de curseur pour l'affichage du curseur */
	private ImageIcon curseur;
	/** Symbole de clef pour l'affichage des lments */
	private ImageIcon clef;
	/** Symbole de coffre pour l'affichage des lments */
	private ImageIcon coffre;
	/** Carte dite */
	private Carte carte;
	
	/** Constructeur d'afficheurCarte utilisant la taille de la carte passe en parametre
	 * pour prendre une taille en consquence */
	public AfficheurCarte(Carte carte, FenetreEditeur fenetre){
		this.setFenetre(fenetre);
		this.setDoubleBuffered(true);
		this.x = 0;
		this.y = 0;
		this.caseX = 0;
		this.caseY = 0;
		this.carte = carte;
		this.hauteur = carte.getLongueur()*16;
		this.largeur = carte.getLargeur()*16;
		this.mode = MODE_AUCUN;
		this.afficherGrille = false;
		this.afficherCurseur = false;
		this.curseur = new ImageIcon("Graphics/Editeur/curseur.png");
		this.clef = new ImageIcon("Graphics/Editeur/clef.png");
		this.coffre = new ImageIcon("Graphics/Editeur/coffre.png");
		
		selectionneur = new SelectionneurMultiple(this);
		deplacementPnj = new ActionDeplacerPnj(carte.getListe(), this);
		
		listeAnnuler = new ListeSauvegardeCase();
		listeRetablir = new ListeSauvegardeCase();
		
		/* l'afficheur detecte quand la souris est utilise sur lui, ici, les clics simples*/
		this.addMouseListener(this);
		/* l'afficheur detecte quand la souris est utilise sur lui, ici le survol de la souris et le clic enfonc/dplac */
		this.addMouseMotionListener(this);
		
	}
	
	/** Methode appele automatiquement par java pour le dessin de l'afficheur */
	public void paintComponent(Graphics g){
		/* on positionne, redimensionne l'afficheur */
		this.setBounds(this.x, this.y, this.largeur, this.hauteur);
		/* on met du blanc comme fond d'afficheur */
		g.setColor(Color.WHITE);
        g.fillRect(0, 0, this.getWidth(), this.getHeight());
        
        /* appel du dessin de la classe Carte */
        carte.paint(g);
        
        afficherGrille(g);
        afficherFranchissabilite(g);
        afficherElement(g);
        
        selectionneur.afficherSelection(g);
        
        afficherCurseur(g);

	}
	
	/** Dessin de la grille*/
	public void afficherGrille(Graphics g){
		if(afficherGrille == true){
			g.setColor(Color.GRAY);
			
			for(int i=1;i<carte.getLargeur();i++)
				g.drawLine(i*16, 0, i*16, carte.getLongueur()*16);
			
			for(int i=1;i<carte.getLongueur();i++)
				g.drawLine(0, i*16, carte.getLargeur()*16, i*16);
		}
	}
	
	/** Entourage en rouge les cases non franchissables */
	public void afficherFranchissabilite(Graphics g){
		if(mode == MODE_FRANCHISSABILITE){
			g.setColor(Color.RED);
			
			for(int i=0;i<carte.getLargeur();i++){
	        	for(int j=0;j<carte.getLongueur();j++){
	        		if(carte.getCase(i, j).isFranchissable() == false)
	        			g.drawRect(i*16, j*16, 15, 15);
	        	}
	        }
		}
	}
	
	/** Mise en valeur le trsor, la cl, les teleporteurs, les indices... */
	public void afficherElement(Graphics g){
		g.setColor(Color.RED);
		if(mode == MODE_ELEMENT){
			for(int i=0;i<carte.getLargeur();i++){
	        	for(int j=0;j<carte.getLongueur();j++){
	        		if(carte.getCase(i, j) == carte.tresor){
	        			g.drawImage(this.coffre.getImage(), i*16, j*16, this);
	        		}else if(carte.getCase(i, j).getClass().getName().equals("org.system.Indice")){
	        			g.drawRect(i*16, j*16, 15, 15);
	        			g.drawString("I", (i*16)+6,(j*16)+12);
		        	}else if(carte.getCase(i, j).getClass().getName().equals("org.system.Clef")){
		        		g.drawImage(this.clef.getImage(), i*16, j*16, this);
		        	}else if(carte.getCase(i, j).getClass().getName().equals("org.system.Incantation")){
		        		g.drawImage(this.clef.getImage(), i*16, j*16, this);
	        		}else if(carte.getCase(i, j).getClass().getName().equals("org.system.Teleporteur")){
	        			g.drawRect(i*16, j*16, 15, 15);
	        			g.drawString("T", (i*16)+4,(j*16)+12);
		        	}
	        	}
	        }
		}
	}
	
	/** Affichage du curseur sur la case survole*/
	public void afficherCurseur(Graphics g){
		if(afficherCurseur == true){
			g.drawImage(curseur.getImage(), caseX*16, caseY*16,null);
		}
	}
	
	/**
	 * Translate l'afficheur avec un vecteur pass en parametre
	 * @param vx dplacement en abcisse
	 * @param vy dplacement en ordonne
	 */
	public void translater(int vx, int vy){
		if((this.x + vx) + this.getWidth() > 0)
			this.x += vx;
		if((this.y + vy) + this.getHeight() > 0)
			this.y += vy;
		repaint();
	}
	
	/**
	 * Dplace l'afficheur  des coordonnes prcises
	 * @param x abcisse
	 * @param y ordonne
	 */
	public void moveTo(int x, int y){
		this.x = x;
		this.y = y;
		repaint();
	}
	
	/** vnement clavier, deplacement de l'afficheur (touches directionnelles) et reset de la position (touche R) */
	public void keyPressed(KeyEvent e) {
		switch(e.getKeyCode()){
			case KeyEvent.VK_LEFT :
				if(e.isControlDown() == false){
					this.translater(-16,0);
				}else{
					if(ListeSauvegardeCase.translater(selectionneur.getListe(), carte, -1, 0) && selectionneur.getP1() != null){
						selectionneur.getP1().x -= 16;
						selectionneur.getP2().x -= 16;
					}
				}
				break;
			case KeyEvent.VK_UP :
				if(e.isControlDown() == false){
					this.translater(0,-16);
				}else{
					if(ListeSauvegardeCase.translater(selectionneur.getListe(), carte, 0, -1) && selectionneur.getP1() != null){
						selectionneur.getP1().y -= 16;
						selectionneur.getP2().y -= 16;
					}
				}
				break;
			case KeyEvent.VK_RIGHT :
				if(e.isControlDown() == false){
					this.translater(16,0);
				}else{
					if(ListeSauvegardeCase.translater(selectionneur.getListe(), carte, 1, 0) && selectionneur.getP1() != null){
						selectionneur.getP1().x += 16;
						selectionneur.getP2().x += 16;
					}
				}
				break;
			case KeyEvent.VK_DOWN :
				if(e.isControlDown() == false){
					this.translater(0,16);
				}else{
					if(ListeSauvegardeCase.translater(selectionneur.getListe(), carte, 0, 1) && selectionneur.getP1() != null){
						selectionneur.getP1().y += 16;
						selectionneur.getP2().y += 16;
					}
				}
				break;
			case KeyEvent.VK_ENTER : 
				ArrayList<SauvegardeCase> tmpListe = new ArrayList<SauvegardeCase>();
				for(int i=0;i<selectionneur.getListe().size();i++){
					tmpListe.add(new SauvegardeCase(carte.getCase(selectionneur.getListe().get(i).getX(), selectionneur.getListe().get(i).getY()), selectionneur.getListe().get(i).getX(), selectionneur.getListe().get(i).getY()));
					carte.getNiveau()[selectionneur.getListe().get(i).getX()][selectionneur.getListe().get(i).getY()] = selectionneur.getListe().get(i).getCase();
				}
				AfficheurCarte.getListeAnnuler().ajouterSauvegarde(tmpListe);
				AfficheurCarte.getListeRetablir().getListeCase().clear();
				fenetre.getBarreEtat().resetAide();
				break;
			case KeyEvent.VK_R :	
				this.moveTo(0, 0);
				break;
			default:
		}
		repaint();
	}
	public void keyReleased(KeyEvent e) {}
	public void keyTyped(KeyEvent e) {}
	
	/* Debut evenement souris */
	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	
	/** Clic de souris gauche simple : selon le mode actuelle on effectue une action spcifique
	 *  Clic de souris droit simple : ou ouvre un menu contextuel pour intragir avec la case clique*/
	public void mousePressed(MouseEvent e) {
		if(e.getButton() == MouseEvent.BUTTON3){
			MenuClicDroit MCD1 = new MenuClicDroit(this, carte, e.getX(), e.getY());
			for(int i=0;i<carte.getListe().Nbelement();i++){
				if(e.getX() >= carte.getListe().getPnj(i).getX() &&
				e.getY() >= carte.getListe().getPnj(i).getY() &&
				e.getX() <= (carte.getListe().getPnj(i).getX()+carte.getListe().getPnj(i).getImageActuelWidth()) &&
				e.getY() <= (carte.getListe().getPnj(i).getY()+carte.getListe().getPnj(i).getImageActuelHeigth())){
					MCD1.ajouterAction(new BoutonSupprimerPnj(carte.getListe().getPnj(i), carte.getListe()));
				}
			}
			MCD1.show(this, e.getX(), e.getY());
		}else{
			/* place une texture sur la case clique */
			if(this.mode == MODE_TEXTURE){
				if(e.isControlDown()){
					PanneauTexture.setIconeTextureSelectionnee(carte.getCase(caseX,caseY).getImageIcon());
					PanneauTexture.setTextureSourceSelectionnee(carte.getCase(caseX,caseY).getSourceImage());
					PanneauTexture.setTextureSelectionnee(carte.getCase(caseX,caseY).getNomImage(), carte.getCase(caseX,caseY).getImageIcon());
				}else if(e.isAltDown() == false){
					if(PanneauTexture.getTextureSourceSelectionnee().equals(carte.getCase(caseX,caseY).getSourceImage()) == false)
						listeAnnuler.ajouterSauvegarde(new SauvegardeCase(carte.getCase(caseX,caseY), caseX, caseY));
					carte.getCase(caseX,caseY).setImageIcon(PanneauTexture.getIconeTextureSelectionnee(), PanneauTexture.getTextureSelectionnee(), PanneauTexture.getTextureSourceSelectionnee());
					listeRetablir.getListeCase().clear();
				}
			}
			/* intervertis la franchissabilit de la case clique */
			else if(this.mode == MODE_FRANCHISSABILITE)
			{
				if(carte.getNiveau()[caseX][caseY].isFranchissable())
					carte.getNiveau()[caseX][caseY].setFranchissable(false);
				else
					carte.getNiveau()[caseX][caseY].setFranchissable(true);
			}
			/* ajoute un pnj (selectionn sur le panneau pnj) a la liste des pnj a l'endroit exact du clic */
			else if(this.mode == MODE_PNJ){
				Pnj pnj = new Pnj(e.getX(), e.getY(), PanneauPnj.getPnjSelectionnee());
				pnj.initVie(1,1,1,1);
					carte.getListe().addPnj(pnj);
			}else if(this.mode == MODE_TRANSPORT){
				System.out.println("Tr:"+PanneauTransport.getTransportSelectionne());
				Transport transport = new Transport(e.getX(), e.getY(), PanneauTransport.getTransportSelectionne(),null);
				int combi[] = {KeyEvent.VK_M};
				transport.setCombinaison(combi);
				transport.setFranchiAir(PanneauTransport.FranchAir[PanneauTransport.indice]);
				transport.setFranchiMer(PanneauTransport.FranchMer[PanneauTransport.indice]);
				transport.setFranchiSol(PanneauTransport.FranchSol[PanneauTransport.indice]);
				carte.getListeTransp().add(transport);
				System.out.println(PanneauTransport.getTransportSelectionne());

			}
		}
		
		/* on redessine immediatement l'afficheur */
		this.repaint();
	}
	
	public void mouseReleased(MouseEvent e) {}
	
	/** Clic de souris enfonc et dplac : on effectue une action en fonction du mode de l'afficheur
	 *  un peu pareil que pour le clic simple mais l'action est prolonge*/
	public void mouseDragged(MouseEvent e) {
		if(clicValide(e) == true){
			caseX = e.getX()/16;
			caseY = e.getY()/16;
			
			/* tant qu'on clique, on rempli d'une texture les cases parcourues */
			if(this.mode == MODE_TEXTURE && e.isAltDown() == false){
				if(PanneauTexture.getTextureSourceSelectionnee().equals(carte.getCase(caseX,caseY).getSourceImage()) == false){
					listeAnnuler.ajouterSauvegarde(new SauvegardeCase(carte.getCase(caseX,caseY), caseX, caseY));
				}
				carte.getCase(caseX,caseY).setImageIcon(PanneauTexture.getIconeTextureSelectionnee(), PanneauTexture.getTextureSelectionnee(), PanneauTexture.getTextureSourceSelectionnee());
				listeRetablir.getListeCase().clear();
			}
			/* si Ctrl en enfonc les case parcourues sont franchissables, sinon, le contraire*/
			else if(this.mode == MODE_FRANCHISSABILITE)
			{
				if(e.isControlDown())
					carte.getNiveau()[caseX][caseY].setFranchissable(true);
				else
					carte.getNiveau()[caseX][caseY].setFranchissable(false);
			}
			this.repaint();
		}
	}

	/** deplacement de la souris : si le curseur est bien dans la zone de travail de l'afficheur
	* on met a jour les coordonnes de la case survole actuellement */
	public void mouseMoved(MouseEvent e) {
		
		if(clicValide(e) == true){
			caseX = e.getX()/16;
			caseY = e.getY()/16;
			
			fenetre.getBarreEtat().setCoordonnee(caseX, caseY);
			
			this.repaint();
		}
	}
	
	/** determine si le curseur est correctement situ sur l'afficheur */
	public boolean clicValide(MouseEvent e){
		if(e.getX() >= 0 && e.getY() >= 0 && e.getX()< this.getWidth() && e.getY()< this.getHeight())
			return true;
		else
			return false;
	}

	/* ****************************** */
	/* PARTIE ACCESSEURS / MODIFIEURS */
	/* ****************************** */
	
	public int getX(){
		return this.x;
	}
	
	public int getY(){
		return this.y;
	}
	
	public void setLargeur(int largeur) {
		this.largeur = largeur;
	}

	public int getLargeur() {
		return largeur;
	}

	public void setHauteur(int hauteur) {
		this.hauteur = hauteur;
	}

	public int getHauteur() {
		return hauteur;
	}
	
	/** si on utilise une nouvelle carte il faut changer la taille de l'afficheur en consquence */
	public void setCarte(Carte carte) {
		selectionneur.reset();
		this.carte = carte;
		this.hauteur = carte.getLongueur()*16;
		this.largeur = carte.getLargeur()*16;
		this.deplacementPnj = new ActionDeplacerPnj(carte.getListe(), this);
		this.repaint();
	}

	public Carte getCarte() {
		return carte;
	}

	public void setMode(int mode) {
		this.mode = mode;
	}

	public int getMode() {
		return mode;
	}
	
	public void setAfficherGrille(boolean afficherGrille) {
		this.afficherGrille = afficherGrille;
	}

	public boolean getAfficherGrille() {
		return afficherGrille;
	}
	
	public void setAfficherCurseur(boolean afficherCurseur) {
		this.afficherCurseur = afficherCurseur;
	}

	public boolean getAfficherCurseur() {
		return afficherCurseur;
	}
	
	public SelectionneurMultiple getSelectionneur(){
		return selectionneur;
	}

	public static ListeSauvegardeCase getListeAnnuler() {
		return listeAnnuler;
	}

	public static ListeSauvegardeCase getListeRetablir() {
		return listeRetablir;
	}

	public void setFenetre(FenetreEditeur fenetre) {
		this.fenetre = fenetre;
	}

	public FenetreEditeur getFenetre() {
		return fenetre;
	}

	public void setDeplacementPnj(ActionDeplacerPnj deplacementPnj) {
		this.deplacementPnj = deplacementPnj;
	}

	public ActionDeplacerPnj getDeplacementPnj() {
		return deplacementPnj;
	}
	
}
