package org.system.newEditor;

import java.awt.event.KeyEvent;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.KeyStroke;

/**
 * Barre de menu. Les lments sont ajouts ici avec leur accesseurs et leur raccourcis
 * On associe les evenement dans des classes anonymes dans FenetreEditeur.java
 * @author Groupe 6 - 2009
 */
public class BarreMenu extends JMenuBar{

	private static final long serialVersionUID = 1L;
	
	private JMenu menuFichier = new JMenu("Fichier");
	private JMenu menuEdition = new JMenu("Edition");
	private JMenu menuAffichage = new JMenu("Affichage");
	private JMenu menuMode = new JMenu("Mode");
	private JMenu menuAide = new JMenu("?");
	
	private JMenuItem actionNouveau = new JMenuItem("Nouveau");
	private JMenuItem actionOuvrir = new JMenuItem("Ouvrir");
	private JMenuItem actionEnregistrer = new JMenuItem("Enregistrer");
	private JMenuItem actionExporterImage = new JMenuItem("Exporter l'image");
	private JMenuItem actionProprieteCarte = new JMenuItem("Proprits de la carte");
	private JMenuItem actionQuitter = new JMenuItem("Quitter");
	
	private JMenuItem actionAnnuler = new JMenuItem("Annuler");
	private JMenuItem actionRetablir = new JMenuItem("Rtablir");
	private JMenuItem actionRemplirCarte = new JMenuItem("Remplir la carte");
	private JMenuItem actionVerifierCarte = new JMenuItem("Vrifier la carte");
	
	private JCheckBoxMenuItem actionAfficherGrille = new JCheckBoxMenuItem("Grille");
	private JCheckBoxMenuItem actionAfficherCurseur = new JCheckBoxMenuItem("Curseur");
	
	private JRadioButtonMenuItem actionActiverModeAucun = new JRadioButtonMenuItem ("Aucun mode");
	private JRadioButtonMenuItem actionActiverFranchissabilite = new JRadioButtonMenuItem ("Franchissabilit");
	private JRadioButtonMenuItem actionPanneauTexture = new JRadioButtonMenuItem("Textures");
	private JRadioButtonMenuItem actionPanneauPnj = new JRadioButtonMenuItem("Pnj");
	private JRadioButtonMenuItem actionPanneauTransport = new JRadioButtonMenuItem("Transport");
	private JRadioButtonMenuItem actionAfficherElement = new JRadioButtonMenuItem("Elments");
	
	private ButtonGroup modeGroup = new ButtonGroup();

	private JMenuItem actionAPropos = new JMenuItem("A propos");
	private JMenuItem actionAide = new JMenuItem("Aide");
	
	public BarreMenu(){
		
		this.menuFichier.add(actionNouveau);
		this.actionNouveau.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, KeyEvent.CTRL_DOWN_MASK));
		
		this.menuFichier.add(actionOuvrir);
		this.actionOuvrir.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.CTRL_DOWN_MASK));
		
		this.menuFichier.add(actionEnregistrer);
		this.actionEnregistrer.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, KeyEvent.CTRL_DOWN_MASK));
		
		this.menuFichier.addSeparator();
		
		this.menuFichier.add(actionExporterImage);
		
		this.menuFichier.addSeparator();
		
		this.menuFichier.add(actionProprieteCarte);
		this.actionProprieteCarte.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, KeyEvent.ALT_DOWN_MASK));
		
		this.menuFichier.addSeparator();
		
		this.menuFichier.add(actionQuitter);
		this.actionQuitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, KeyEvent.CTRL_DOWN_MASK));
		
		this.menuEdition.add(actionAnnuler);
		this.actionAnnuler.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, KeyEvent.CTRL_DOWN_MASK));
		this.menuEdition.add(actionRetablir);
		this.actionRetablir.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Y, KeyEvent.CTRL_DOWN_MASK));
		this.menuEdition.addSeparator();
		this.menuEdition.add(actionRemplirCarte);
		this.actionRemplirCarte.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, KeyEvent.CTRL_DOWN_MASK));
		this.menuEdition.add(actionVerifierCarte);
		this.actionVerifierCarte.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, KeyEvent.CTRL_DOWN_MASK));
		
		
		
		this.menuAffichage.add(actionAfficherGrille);
		this.actionAfficherGrille.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, KeyEvent.CTRL_DOWN_MASK));
		this.menuAffichage.add(actionAfficherCurseur);
		this.actionAfficherCurseur.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, KeyEvent.CTRL_DOWN_MASK));
		
		this.modeGroup.add(actionActiverModeAucun);
		this.modeGroup.add(actionActiverFranchissabilite);
		this.modeGroup.add(actionPanneauTexture);
		this.modeGroup.add(actionPanneauPnj);
		this.modeGroup.add(actionPanneauTransport);
		this.modeGroup.add(actionAfficherElement);
		
		this.menuMode.add(actionActiverModeAucun);
		actionActiverModeAucun.setSelected(true);
		
		this.menuMode.add(actionActiverFranchissabilite);
		this.actionActiverFranchissabilite.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, KeyEvent.CTRL_DOWN_MASK));
		
		this.menuMode.add(actionPanneauTexture);
		this.actionPanneauTexture.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, KeyEvent.CTRL_DOWN_MASK));
		
		this.menuMode.add(actionPanneauPnj);
		this.actionPanneauPnj.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, KeyEvent.CTRL_DOWN_MASK));
		
		this.menuMode.add(actionPanneauTransport);
		
		this.menuMode.add(actionAfficherElement);
		this.actionAfficherElement.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, KeyEvent.CTRL_DOWN_MASK));
		
		this.menuAide.add(actionAPropos);
		this.menuAide.add(actionAide);
		
		this.add(menuFichier);
		this.add(menuEdition);
		this.add(menuAffichage);
		this.add(menuMode);
		this.add(menuAide);
	}
	
	public JMenuItem getActionNouveau(){
		return this.actionNouveau;
	}

	public JRadioButtonMenuItem getActionPanneauTexture() {
		return actionPanneauTexture;
	}
	
	public JMenuItem getActionQuitter(){
		return this.actionQuitter;
	}


	public JCheckBoxMenuItem getActionAfficherGrille() {
		return actionAfficherGrille;
	}

	public JRadioButtonMenuItem getActionActiverFranchissabilite() {
		return actionActiverFranchissabilite;
	}

	public JRadioButtonMenuItem getActionPanneauPnj() {
		return actionPanneauPnj;
	}

	public JMenuItem getActionEnregistrer() {
		return actionEnregistrer;
	}

	public JMenuItem getActionOuvrir() {
		return actionOuvrir;
	}

	public JCheckBoxMenuItem getActionAfficherCurseur() {
		return actionAfficherCurseur;
	}

	public JRadioButtonMenuItem getActionActiverModeAucun() {
		return actionActiverModeAucun;
	}

	public JRadioButtonMenuItem getActionAfficherElement() {
		return actionAfficherElement;
	}

	public JMenuItem getActionAPropos() {
		return actionAPropos;
	}

	public JMenuItem getActionExporterImage() {
		return actionExporterImage;
	}

	public JMenuItem getActionProprieteCarte() {
		return actionProprieteCarte;
	}

	public JMenuItem getActionAnnuler() {
		return actionAnnuler;
	}

	public JMenuItem getActionVerifierCarte() {
		return actionVerifierCarte;
	}

	public JRadioButtonMenuItem getActionPanneauTransport() {
		return actionPanneauTransport;
	}

	public JMenuItem getActionRetablir() {
		return actionRetablir;
	}

	public JMenuItem getActionRemplirCarte() {
		return actionRemplirCarte;
	}

	public JMenuItem getActionAide() {
		return actionAide;
	}
}
