package org.system.newEditor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.system.Carte;

/**
 * Une boite de dialogue qui permet d'exporter la carte en un fichier .map dans le dossier "maps"
 * @author Groupe 6 - 2009
 */
public class DialogEnregistrer extends JDialog {
	
	private static final long serialVersionUID = 1L;
	
	private JButton okBouton, annulerBouton;
	private JLabel statusLabel;
	private JTextField nomField;
	
	private Carte carte;
	
	public DialogEnregistrer(JFrame parent, Carte carte){
		super(parent, "Enregistrer la carte", true);
		this.setSize(250, 170);
		this.setLocationRelativeTo(null);
		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		this.carte = carte;
		
		this.initialiserComposant();
		
		this.setVisible(true);
	}
	
	public boolean confirmerSiFichierExiste(String filename){
		File fichier = new File("maps/"+filename+".map");
		if(fichier.exists()){
			int option = JOptionPane.showConfirmDialog(null, "Le fichier "+filename+" existe dj. Voulez vous l'craser?", "Confirmer l'enregistrement", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
			
			if(option == 1) return true;
		}
		
		return false;
	}
	
	public void initialiserComposant(){
		// zone nom carte
		nomField = new JTextField(carte.getNomCarte());
		JPanel nomPanel = new JPanel();
		
		nomField.setPreferredSize(new Dimension(200, 20));
		nomPanel.setPreferredSize(new Dimension(220, 60));
		nomPanel.setBorder(BorderFactory.createTitledBorder("Nom du fichier"));
		
		nomPanel.add(nomField);
		
		// zone status
		statusLabel = new JLabel("");
		JPanel statusPanel = new JPanel();
		statusPanel.setPreferredSize(new Dimension(220, 30));

		statusPanel.add(statusLabel);
		
		// zone bouton
		okBouton = new JButton("Enregistrer");
		annulerBouton = new JButton("Annuler");
		JPanel boutonPanel = new JPanel();
		
		boutonPanel.add(okBouton);
		boutonPanel.add(annulerBouton);
		
		// conteneur principal
		JPanel conteneurPanel = new JPanel();
		conteneurPanel.add(nomPanel);
		conteneurPanel.add(statusPanel);
		
		okBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg) {
				
				String nomCarte = nomField.getText();
				
				if(nomCarte.length() > 0){
					
					if(confirmerSiFichierExiste(nomCarte) == false){
						FileOutputStream fichier = null;
						ObjectOutputStream oos = null;
						
						try {
							carte.setNomCarte(nomCarte);
							fichier = new FileOutputStream("maps/"+nomCarte+".map");
							
							oos = new ObjectOutputStream(fichier);
							oos.writeObject(carte);
							oos.close();
							
						} catch (FileNotFoundException e1) {
							e1.printStackTrace();
						} catch (IOException e2) {
							e2.printStackTrace();
						}
					}
					
					setVisible(false);
				}
				else{
					statusLabel.setForeground(Color.RED);
					statusLabel.setText("Nom de la carte incorrect");
				}
				
			}			
		});
		
		annulerBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}			
		});
		
		this.getContentPane().add(conteneurPanel, BorderLayout.CENTER);
		this.getContentPane().add(boutonPanel, BorderLayout.SOUTH);
	}
}
