package org.system.newEditor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.system.Carte;

/**
 *	Boite de dialogue pour exporter l'image de la carte dans le dossier "savedThemes" 
 * @author Groupe 6 - 2009
 */
public class DialogExporterImage extends JDialog{

	private static final long serialVersionUID = 1L;
	
	private Carte carte;
	private JLabel extensionLabel, nomImageLabel, statusLabel;
	private JComboBox extensionBox;
	private JTextField nomImageField;
	private JButton okBouton, annulerBouton;
	
	public DialogExporterImage(JFrame parent, Carte carte){
		super(parent, "Exporter l'image", true);
		this.setSize(300, 180);
		this.setLocationRelativeTo(null);
		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		this.carte = carte;
		
		this.initialiserComposant();
		
		this.setVisible(true);
	}

	public void initialiserComposant(){
		
		// zone extension
		extensionBox = new JComboBox();
		extensionBox.addItem("png");
		extensionBox.addItem("jpeg");
		extensionLabel = new JLabel("Format de l'image : ");
		JPanel extensionPanel = new JPanel();
		
		extensionPanel.add(extensionLabel);
		extensionPanel.add(extensionBox);
		extensionPanel.setPreferredSize(new Dimension(280, 40));
		
		// zone nomImage
		nomImageField = new JTextField(carte.getNomCarte());
		nomImageField.setPreferredSize(new Dimension(170, 25));

		nomImageLabel = new JLabel("Nom du fichier : ");
		JPanel nomImagePanel = new JPanel();
		
		nomImagePanel.add(nomImageLabel);
		nomImagePanel.add(nomImageField);
		nomImagePanel.setPreferredSize(new Dimension(280, 30));
		
		// zone status
		statusLabel = new JLabel("");
		statusLabel.setForeground(Color.RED);
		JPanel statusPanel = new JPanel();
		statusPanel.setPreferredSize(new Dimension(220, 30));

		statusPanel.add(statusLabel);
		
		// zone bouton
		okBouton = new JButton("Enregistrer");
		annulerBouton = new JButton("Annuler");
		JPanel boutonPanel = new JPanel();
		
		boutonPanel.add(okBouton);
		boutonPanel.add(annulerBouton);
		
		// conteneur principal
		JPanel conteneurPanel = new JPanel();
		conteneurPanel.add(extensionPanel);
		conteneurPanel.add(nomImagePanel);
		conteneurPanel.add(statusPanel);
		
		okBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg) {
				String nomImage = nomImageField.getText();
				
				if(nomImage.length() > 0){
					File file = new File("savedThemes/"+nomImage+"."+extensionBox.getSelectedItem().toString());
					
					BufferedImage image = new BufferedImage(carte.getLongueur()*16, carte.getLargeur()*16, BufferedImage.TYPE_INT_RGB);
			    	
					Graphics2D g2 = image.createGraphics();
			    	carte.paint(g2);
			    	g2.dispose();
			    	
			    	try {
						ImageIO.write(image, extensionBox.getSelectedItem().toString(), file);
						setVisible(false);
					} catch (IOException e1) {		
						e1.printStackTrace();
						statusLabel.setText("Erreur lors de la cration de l'image");
					}
				}
				else{
					statusLabel.setText("Nom de l'image incorrect");
				}
			}
		});
		
		
		annulerBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}			
		});
		
		this.getContentPane().add(conteneurPanel, BorderLayout.CENTER);
		this.getContentPane().add(boutonPanel, BorderLayout.SOUTH);
	}
}
