package org.system.newEditor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.system.Carte;
import org.system.Indice;

/**
 * Fenetre permettant de positionner un indice, en demandant une combinaison
 * @author Groupe 6 - 2009
 */
public class DialogIndice extends JDialog{

	private static final long serialVersionUID = 1L;
	
	private JButton okBouton, annulerBouton;
	private JTextField indiceField;
	private JLabel indiceLabel;
	private JLabel statusLabel;
	private PanneauCombinaison combinaisonPanneau;
	
	private Carte carte;
	private int caseX;
	private int caseY;
	
	public DialogIndice(JFrame parent, Carte carte, int caseX, int caseY, boolean modifier){
		super(parent, "", true);
		this.setSize(400, 240);
		this.setLocationRelativeTo(null);
		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		this.carte = carte;
		this.caseX = caseX;
		this.caseY = caseY;
		
		this.initialiserComposant(modifier);
		
		this.setVisible(true);
	}
	
	public void initialiserComposant(boolean modifier){
		
		combinaisonPanneau = new PanneauCombinaison("Entrez la combinaison de touches pour activer l'indice :", 3, this.getWidth());

		if(modifier == true){
			this.setTitle("Modifier un indice");
			indiceField = new JTextField(((Indice) carte.getCase(caseX, caseY)).getMessage());
			okBouton = new JButton("Modifier");
			
			combinaisonPanneau.setCombinaison(carte.getCase(caseX, caseY).getCombinaison());
		}
		else{
			this.setTitle("Ajouter un indice");
			indiceField = new JTextField();
			okBouton = new JButton("Creer");
		}
		
		//zone texte indice
		JPanel indicePanel = new JPanel();
		indicePanel.setPreferredSize(new Dimension(380, 60));
		
		indiceField.setPreferredSize(new Dimension(360, 25));
		
		indiceLabel = new JLabel("Veuillez entrer l'indice :");
		
		indicePanel.add(indiceLabel);
		indicePanel.add(indiceField);
		
		// zone status
		statusLabel = new JLabel("");
		statusLabel.setForeground(Color.RED);
		JPanel statusPanel = new JPanel();
		statusPanel.setPreferredSize(new Dimension(360, 30));

		statusPanel.add(statusLabel);
		
		// zone bouton
		annulerBouton = new JButton("Annuler");
		JPanel boutonPanel = new JPanel();
		
		boutonPanel.add(okBouton);
		boutonPanel.add(annulerBouton);
		
		//conteneur principal
		JPanel conteneurPanel = new JPanel();
		conteneurPanel.add(indicePanel);
		conteneurPanel.add(combinaisonPanneau);
		conteneurPanel.add(statusPanel);
		
		annulerBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}			
		});
		
		okBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(indiceField.getText().length() > 0){
					if(combinaisonPanneau.getCombinaison() != null){
						carte.getNiveau()[caseX][caseY] = new Indice(carte.getCase(caseX, caseY), indiceField.getText(), combinaisonPanneau.getCombinaison());
						setVisible(false);
					}else {
						statusLabel.setText("Veuillez entrer une combinaison");
					}	
				}else {
					statusLabel.setText("Veuillez entrer un indice");
				}
			}			
		});
		
		
	
		this.getContentPane().add(conteneurPanel, BorderLayout.CENTER);
		this.getContentPane().add(boutonPanel, BorderLayout.SOUTH);
	}
	
	public int[] obtenirCombinaison(JComboBox[] comboBox){
		int longueurCombinaison = 0;
		int i, j;
		
		for(i=0;i<3;i++){
			for(j=0;j<3;j++){
				if(i != j)
					if(comboBox[i].getSelectedItem().toString().charAt(0) == comboBox[j].getSelectedItem().toString().charAt(0))
						comboBox[j].setSelectedIndex(0);
			}
		}
		
		for(i=0;i<3;i++)
			if(comboBox[i].getSelectedItem().toString() != " ")
				longueurCombinaison++;
		
		if(longueurCombinaison == 0) return null;
		
		int[] combinaison = new int[longueurCombinaison];
		j=0;
		for(i=0;i<3;i++){
			if(comboBox[i].getSelectedItem().toString() != " "){
				combinaison[j] = (int)comboBox[i].getSelectedItem().toString().charAt(0);
				j++;
			}
		}
		
		return combinaison;
	}

}
