
package org.system.newEditor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.system.Carte;

/**
 * Bote de dialogue permettant de crer une nouvelle carte
 * L'utilisateur indique la taille
 * @author Groupe 6 - 2009
 */
public class DialogNouveau extends JDialog{

	private static final long serialVersionUID = 1L;
	
	private JButton okBouton, annulerBouton;
	private JLabel statusLabel, tailleXLabel, tailleYLabel;
	private JTextField largeurField, hauteurField, nombreEquipeField, nomCarteField;
	private JComboBox textureBox;
	
	private Carte carte;
	
	/**
	 * Constructeur de la boite de dialog "Nouveau"
	 * @param parent la JFrame qui appelle le constructeur
	 */
	public DialogNouveau(JFrame parent){
		/* appel au constructeur de la classe parente, JFrame */
		super(parent, "Nouvelle carte", true);
		this.setSize(400, 270);
		this.setLocationRelativeTo(null);
		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		this.initialiserComposant();
		
		this.setVisible(true);
	}
	
	/**
	 * Place les composants dans la fenetre
	 * Dfinit l'action du bouton OK
	 */
	public void initialiserComposant(){
		carte = null;
		
		// zone nombreEquipe
		nombreEquipeField = new JTextField("2");
		nombreEquipeField.setPreferredSize(new Dimension(60, 30));
		
		JPanel nombreEquipePanel = new JPanel();
		nombreEquipePanel.setPreferredSize(new Dimension(190, 90));
		nombreEquipePanel.setBorder(BorderFactory.createTitledBorder("Nombre d'quipes"));
		
		nombreEquipePanel.add(nombreEquipeField);
		
		// taille carte
		tailleXLabel = new JLabel("Largeur de la carte");
		tailleYLabel = new JLabel("Hauteur de la carte");
		largeurField = new JTextField("20");
		hauteurField = new JTextField("20");
		
		largeurField.setPreferredSize(new Dimension(60, 25));
		hauteurField.setPreferredSize(new Dimension(60, 25));
		JPanel taillePanel = new JPanel();
		taillePanel.setPreferredSize(new Dimension(190, 90));
		taillePanel.setBorder(BorderFactory.createTitledBorder("Taille de la carte"));
		
		taillePanel.add(tailleXLabel);
		taillePanel.add(largeurField);
		taillePanel.add(tailleYLabel);
		taillePanel.add(hauteurField);
		
		//zone texture remplissage
		textureBox = new JComboBox();
		for(int i=0;i<FenetreEditeur.getListeTextureBasique().getLongueur();i++){
			textureBox.addItem(FenetreEditeur.getListeTextureBasique().getListe().get(i).getImageIcon());
		}

		JPanel texturePanel = new JPanel();
		texturePanel.setPreferredSize(new Dimension(190, 70));
		texturePanel.setBorder(BorderFactory.createTitledBorder("Texture"));
		texturePanel.add(textureBox);
		
		// zone nom carte
		nomCarteField = new JTextField("Sans titre");
		nomCarteField.setPreferredSize(new Dimension(150, 25));
		
		JPanel nomCartePanel = new JPanel();
		nomCartePanel.setPreferredSize(new Dimension(190, 70));
		nomCartePanel.setBorder(BorderFactory.createTitledBorder("Nom de la carte"));
		
		nomCartePanel.add(nomCarteField);

		// zone status
		statusLabel = new JLabel("");
		statusLabel.setForeground(Color.RED);
		
		JPanel statusPanel = new JPanel();
		statusPanel.setPreferredSize(new Dimension(380, 30));

		statusPanel.add(statusLabel);
		
		// zone bouton
		okBouton = new JButton("Creer");
		annulerBouton = new JButton("Annuler");
		JPanel boutonPanel = new JPanel();
		
		boutonPanel.add(okBouton);
		boutonPanel.add(annulerBouton);
		
		// conteneur principal
		JPanel conteneurPanel = new JPanel();
		conteneurPanel.add(nombreEquipePanel);
		conteneurPanel.add(taillePanel);
		conteneurPanel.add(nomCartePanel);
		conteneurPanel.add(texturePanel);
		conteneurPanel.add(statusPanel);
		
		this.getContentPane().add(conteneurPanel, BorderLayout.CENTER);
		this.getContentPane().add(boutonPanel, BorderLayout.SOUTH);
		
		
		annulerBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}			
		});
		
		okBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg) {
				
				/* Renvoie une exception si les valeurs tapes ne sont pas des chiffres (vide, lettres...)
				 * sauf pour le nom de la carte */
				try{
					int largeur = Integer.parseInt(largeurField.getText());
					int hauteur = Integer.parseInt(hauteurField.getText());
					int nombreEquipe = Integer.parseInt(nombreEquipeField.getText());
					String nomCarte = nomCarteField.getText();
					
					if(largeur >= 10 && hauteur >= 10 && largeur <= 100 && hauteur <= 100){
						if(nombreEquipe > 1 && nombreEquipe < 10){
							if(nomCarte.length() > 0){
								statusLabel.setText("");
								
								/* cration d'une carte de la taille voulue
								 * Modifie le nombre d'quipes, et le nom de la carte */
								carte = new Carte(largeur, hauteur, null);
							//	carte.setNombreEquipe(nombreEquipe);
								carte.setNomCarte(nomCarte);
								
								for(int i=0;i<largeur;i++)	{
									for(int j=0;j<hauteur;j++)	{
										carte.getCase(i, j).setVisible(true);
										carte.getCase(i, j).setTexture(FenetreEditeur.getListeTextureBasique().getListe().get(textureBox.getSelectedIndex()));
									}
								}
								
								setVisible(false);
							}
							else{
								statusLabel.setText("Nom de la carte incorrect");
							}	
						}
						else{
							statusLabel.setText("Nombre d'quipes incorrect");
						}
					}
					else{
						statusLabel.setText("Taille de la carte incorrecte");
					}
					
				} catch(NumberFormatException e){
					statusLabel.setText("Un des champs n'a pas t rempli");
				}
				
			}			
		});
	}
	
	/**
	 * @return carte cre par l'utilisateur
	 */
	public Carte getCarte(){
		return this.carte;
	}
}
