package org.system.newEditor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.ObjectInputStream;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import org.system.Carte;

/**
 * Fenetre permettant d'ouvrir et charger une carte dans l'diteur
 * @author Groupe 6 - 2009
 */
public class DialogOuvrir extends JDialog {
	
	private static final long serialVersionUID = 1L;
	
	private JButton okNomSaisiBouton, annulerBouton;
	private JLabel statusLabel;
	private JComboBox fichierBox;
	
	private Carte carte;
	
	public DialogOuvrir(JFrame parent){
		super(parent, "Ouvrir", true);
		this.setSize(250, 170);
		this.setLocationRelativeTo(null);
		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		this.initialiserComposant();
		
		this.setVisible(true);
	}
	
	public void initialiserComposant(){
		carte = null;
		int i;
		
		// zone status
		statusLabel = new JLabel("");
		statusLabel.setForeground(Color.RED);
		JPanel statusPanel = new JPanel();
		statusPanel.setPreferredSize(new Dimension(220, 30));

		statusPanel.add(statusLabel);
		
		//zone liste fichiers
		fichierBox = new JComboBox();
		fichierBox.addItem("");
		File repertoire = new File("maps/");
		String [] listeFichier;
		listeFichier=repertoire.list();
		for(i=0;i<listeFichier.length;i++)
		{
			if(listeFichier[i].endsWith(".map")){
				fichierBox.addItem(listeFichier[i].substring(0, listeFichier[i].indexOf(".map")));
			}
		}
		
		JPanel fichierPanel = new JPanel();
		fichierPanel.add(fichierBox);
		
		fichierPanel.setPreferredSize(new Dimension(220, 60));
		fichierPanel.setBorder(BorderFactory.createTitledBorder("Fichiers existants"));
		
		// zone bouton
		okNomSaisiBouton = new JButton("Ouvrir");
		annulerBouton = new JButton("Annuler");
		JPanel boutonPanel = new JPanel();
		
		boutonPanel.add(okNomSaisiBouton);
		boutonPanel.add(annulerBouton);
		
		// conteneur principal
		JPanel conteneurPanel = new JPanel();
		conteneurPanel.add(fichierPanel);
		conteneurPanel.add(statusPanel);
		
		okNomSaisiBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg) {
				
				String nomCarte = fichierBox.getSelectedItem().toString();
				
				if(nomCarte.length() > 0){
					if(new File("maps/"+nomCarte+".map").exists()){
						
						FileInputStream fichier = null;
						ObjectInputStream oos = null;
						
						try {
							fichier = new FileInputStream("maps/"+nomCarte+".map");
							oos = new ObjectInputStream(fichier);
							carte = (Carte) oos.readObject();
							oos.close();
							
						} catch (FileNotFoundException e1) {
							e1.printStackTrace();
						} catch (IOException e2) {
							e2.printStackTrace();
						} catch (ClassNotFoundException e3) {
							e3.printStackTrace();
						}
						
						synchroniserImage();
						setVisible(false);
					}else{
						statusLabel.setText("Fichier inexistant");
					}
				}else{
					statusLabel.setText("Veuillez entrer un nom de fichier");
				}
			}			
		});
		
		annulerBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}			
		});
		
		this.getContentPane().add(conteneurPanel, BorderLayout.CENTER);
		this.getContentPane().add(boutonPanel, BorderLayout.SOUTH);
	}
	
	/**
	 * Pour chaque case on regarde si le lien vers l'image "Graphics/texture/..." est identique
	 * a une des textures toutes les listes de textures.
	 * Si c'est le cas, on remplace la rfrence de l'ImageIcon de la case vers celle de la liste de texture.
	 * Avantages : Une unique rfrence par image, fichier .map rduit. En changeant une image
	 * dans le dossier "Graphics", l'image sera change dans la carte.
	 */
	public void synchroniserImage(){
		int i, j;
		
		for(i=0;i<carte.getLargeur();i++){
        	for(j=0;j<carte.getLongueur();j++){
        		if(carte.getCase(i, j).getSourceImage() != null){
        			if(FenetreEditeur.getListeTextureMontagneTerrain().contient(carte.getCase(i, j).getSourceImage())){
        				carte.getCase(i, j).setImageIcon(FenetreEditeur.getListeTextureMontagneTerrain().trouveBySource(carte.getCase(i, j).getSourceImage()).getImageIcon(), carte.getCase(i, j).getNomImage(), carte.getCase(i, j).getSourceImage());
        			}
        			else if(FenetreEditeur.getListeTextureMontagneObstacle().contient(carte.getCase(i, j).getSourceImage())){
        				carte.getCase(i, j).setImageIcon(FenetreEditeur.getListeTextureMontagneObstacle().trouveBySource(carte.getCase(i, j).getSourceImage()).getImageIcon(), carte.getCase(i, j).getNomImage(), carte.getCase(i, j).getSourceImage());
        			}
        			else if(FenetreEditeur.getListeTextureMontagneBatiment().contient(carte.getCase(i, j).getSourceImage())){
        				carte.getCase(i, j).setImageIcon(FenetreEditeur.getListeTextureMontagneBatiment().trouveBySource(carte.getCase(i, j).getSourceImage()).getImageIcon(), carte.getCase(i, j).getNomImage(), carte.getCase(i, j).getSourceImage());
        			}
        			else if(FenetreEditeur.getListeTextureTropicalTerrain().contient(carte.getCase(i, j).getSourceImage())){
        				carte.getCase(i, j).setImageIcon(FenetreEditeur.getListeTextureTropicalTerrain().trouveBySource(carte.getCase(i, j).getSourceImage()).getImageIcon(), carte.getCase(i, j).getNomImage(), carte.getCase(i, j).getSourceImage());
        			}
        			else if(FenetreEditeur.getListeTextureTropicalObstacle().contient(carte.getCase(i, j).getSourceImage())){
        				carte.getCase(i, j).setImageIcon(FenetreEditeur.getListeTextureTropicalObstacle().trouveBySource(carte.getCase(i, j).getSourceImage()).getImageIcon(), carte.getCase(i, j).getNomImage(), carte.getCase(i, j).getSourceImage());
        			}
        			else if(FenetreEditeur.getListeTextureTropicalBatiment().contient(carte.getCase(i, j).getSourceImage())){
        				carte.getCase(i, j).setImageIcon(FenetreEditeur.getListeTextureTropicalBatiment().trouveBySource(carte.getCase(i, j).getSourceImage()).getImageIcon(), carte.getCase(i, j).getNomImage(), carte.getCase(i, j).getSourceImage());
        			}
        			else if(FenetreEditeur.getListeTextureChateauTerrain().contient(carte.getCase(i, j).getSourceImage())){
        				carte.getCase(i, j).setImageIcon(FenetreEditeur.getListeTextureChateauTerrain().trouveBySource(carte.getCase(i, j).getSourceImage()).getImageIcon(), carte.getCase(i, j).getNomImage(), carte.getCase(i, j).getSourceImage());
        			}
        			else if(FenetreEditeur.getListeTextureChateauObstacle().contient(carte.getCase(i, j).getSourceImage())){
        				carte.getCase(i, j).setImageIcon(FenetreEditeur.getListeTextureChateauObstacle().trouveBySource(carte.getCase(i, j).getSourceImage()).getImageIcon(), carte.getCase(i, j).getNomImage(), carte.getCase(i, j).getSourceImage());
        			}
        		}
        	}
		}
	}
	
	/** La carte ouverte est rcuprable */
	public Carte getCarte(){
		return this.carte;
	}
}
