package org.system.newEditor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.system.Carte;

/**
 * Fenetre affichant des infos sur la carte
 * et permettant de saisir les dpart des quipes
 * @author Groupe 6 - 2009
 */
public class DialogProprieteCarte extends JDialog{

	private static final long serialVersionUID = 1L;
	
	private Carte carte;
	
	private JLabel statusLabel;
	private JLabel[] labelDepart;
	private JTextField[] departXField, departYField;
	private JButton okBouton, annulerBouton;
	
	public DialogProprieteCarte(JFrame parent, Carte carte){
		super(parent, "Proprits de la carte", false);
		this.setSize(300, 200+(carte.getEquipeInfo().getNombreEquipe()*30));
		this.setLocationRelativeTo(null);
		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		this.carte = carte;
		
		this.initialiserComposant();
		
		this.setVisible(true);
	}

	public void initialiserComposant(){
		int nbEquipe = carte.getEquipeInfo().getNombreEquipe();
		
		//zone info carte
		JPanel panelInfoCarte = new JPanel();
		panelInfoCarte.setPreferredSize(new Dimension(130, 70));
		
		panelInfoCarte.add(new JLabel("Largeur de la carte: "+carte.largeur));
		panelInfoCarte.add(new JLabel("Hauteur de la carte: "+carte.longueur));
		panelInfoCarte.add(new JLabel("Nombre d'quipes : "+nbEquipe));
		
		
		//nb equipes
		JPanel panelNombreEquipe = new JPanel();
		JLabel labelNombreEquipe = new JLabel("Case dpart des quipes :");
		panelNombreEquipe.setPreferredSize(new Dimension(280, 20));
		panelNombreEquipe.add(labelNombreEquipe);
		
		// depart de chaque team
		JPanel panelDepartEquipe = new JPanel();
		panelDepartEquipe.setPreferredSize(new Dimension(280, carte.getEquipeInfo().getNombreEquipe()*30));
		departXField = new JTextField[nbEquipe];
		departYField = new JTextField[nbEquipe];
		labelDepart = new JLabel[nbEquipe];
		
		for(int i=0;i<nbEquipe; i++){
			labelDepart[i] = new JLabel("Equipe " + (i+1) + " : X ");
			departXField[i] = new JTextField((int)carte.getEquipeInfo().getDepart(i).getX()+"");
			departXField[i].setPreferredSize(new Dimension(55, 25));
			
			departYField[i] = new JTextField((int)carte.getEquipeInfo().getDepart(i).getY()+"");
			departYField[i].setPreferredSize(new Dimension(55, 25));
			
			panelDepartEquipe.add(labelDepart[i]);
			panelDepartEquipe.add(departXField[i]);
			panelDepartEquipe.add(new JLabel(" Y "));
			panelDepartEquipe.add(departYField[i]);
		}
		
		// zone status
		statusLabel = new JLabel();
		statusLabel.setForeground(Color.RED);
		
		JPanel statusPanel = new JPanel();
		statusPanel.setPreferredSize(new Dimension(280, 30));
		statusPanel.add(statusLabel);
		
		// zone bouton
		okBouton = new JButton("Ok");
		annulerBouton = new JButton("Annuler");
		JPanel boutonPanel = new JPanel();
		
		boutonPanel.add(okBouton);
		boutonPanel.add(annulerBouton);
		
		
		// regroupement
		JPanel conteneurPanel = new JPanel();
		conteneurPanel.add(panelInfoCarte);
		conteneurPanel.add(panelNombreEquipe);
		conteneurPanel.add(panelDepartEquipe);
		conteneurPanel.add(statusPanel);
		
		annulerBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}			
		});
		
		okBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				try{
					boolean valide = true;
					for(int i=0;i<carte.getEquipeInfo().getNombreEquipe(); i++){
						if(Integer.parseInt(departXField[i].getText()) < 0 || Integer.parseInt(departXField[i].getText()) >= carte.getLongueur()
								|| Integer.parseInt(departYField[i].getText()) < 0 || Integer.parseInt(departYField[i].getText()) >= carte.getLargeur())
							valide = false;
					}
					
					if(valide == true){
						for(int i=0;i<carte.getEquipeInfo().getNombreEquipe(); i++){
							carte.getEquipeInfo().setDepart(i, Integer.parseInt(departXField[i].getText()),
																Integer.parseInt(departYField[i].getText()));
						}
						
						setVisible(false);
					}else{
						statusLabel.setText("Une/des coordonnes ne sont pas correctes");
					}
					
				} catch(NumberFormatException e){
					statusLabel.setText("Un des champs n'a pas t rempli");
				}
			}			
		});
		
		this.getContentPane().add(conteneurPanel, BorderLayout.CENTER);
		this.getContentPane().add(boutonPanel, BorderLayout.SOUTH);
	}
}
