package org.system.newEditor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.system.Carte;
import org.system.Teleporteur;

/**
 * Fenetre permettant de positionner un tlporteur, en demandant une combinaison et les coordonnes de la case d'arrive
 * @author Groupe 6 - 2009
 */
public class DialogTeleporteur extends JDialog{

	private static final long serialVersionUID = 1L;
	
	private JButton okBouton, annulerBouton;

	private JLabel statusLabel;
	private PanneauCombinaison combinaisonPanneau;
	private JTextField arriveeX, arriveeY;
	
	private Carte carte;
	private int caseX;
	private int caseY;
	
	public DialogTeleporteur(JFrame parent, Carte carte, int caseX, int caseY, boolean modifier){
		super(parent, "", true);
		this.setSize(400, 210);
		this.setLocationRelativeTo(null);
		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		this.carte = carte;
		this.caseX = caseX;
		this.caseY = caseY;
		
		this.initialiserComposant(modifier);
		
		this.setVisible(true);
	}
	
	public void initialiserComposant(boolean modifier){
		
		combinaisonPanneau = new PanneauCombinaison("Entrez la combinaison de touches pour activer le tlporteur :", 3, this.getWidth());
		arriveeX = new JTextField();
		arriveeY = new JTextField();
		
		if(modifier == true){
			okBouton = new JButton("Modifier un tlporteur");
			
			/* On met a jour avec les combinaisons actuelle */
			combinaisonPanneau.setCombinaison(carte.getCase(caseX, caseY).getCombinaison());
			arriveeX.setText(((Teleporteur) carte.getCase(caseX, caseY)).getArriverX()/16 + "");
			arriveeY.setText(((Teleporteur) carte.getCase(caseX, caseY)).getArriverY()/16 + "");
		}
		else{
			this.setTitle("Crer un tlporteur");
			okBouton = new JButton("Poser");
		}
		
		//zone arrivee
		arriveeX.setPreferredSize(new Dimension(50, 25));
		arriveeY.setPreferredSize(new Dimension(50, 25));
		JPanel arriveePanel = new JPanel();
		arriveePanel.setPreferredSize(new Dimension(360, 35));

		arriveePanel.add(new JLabel("Case d'arrive"));
		arriveePanel.add(new JLabel(" X : "));
		arriveePanel.add(arriveeX);
		arriveePanel.add(new JLabel(" Y : "));
		arriveePanel.add(arriveeY);
		
		// zone status
		statusLabel = new JLabel("");
		statusLabel.setForeground(Color.RED);
		JPanel statusPanel = new JPanel();
		statusPanel.setPreferredSize(new Dimension(360, 30));

		statusPanel.add(statusLabel);
		
		// zone bouton
		annulerBouton = new JButton("Annuler");
		JPanel boutonPanel = new JPanel();
		
		boutonPanel.add(okBouton);
		boutonPanel.add(annulerBouton);
		
		//conteneur principal
		JPanel conteneurPanel = new JPanel();
		conteneurPanel.add(arriveePanel);
		conteneurPanel.add(combinaisonPanneau);
		conteneurPanel.add(statusPanel);
		
		annulerBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				setVisible(false);
			}			
		});
		
		okBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				if(combinaisonPanneau.getCombinaison() != null){
					try{
						int aX = Integer.parseInt(arriveeX.getText());
						int aY = Integer.parseInt(arriveeY.getText());
						
						if(aX >= 0 && aY >= 0 && aX < carte.getLongueur() && aY < carte.getLargeur()){
							if(carte.getCase(caseX,caseY) != carte.getCase(aX,aY)){
								carte.getNiveau()[caseX][caseY] = new Teleporteur(carte.getCase(caseX,caseY), combinaisonPanneau.getCombinaison(), new Point(aX*16,aY*16));
								setVisible(false);
							}else
								statusLabel.setText("Coordonnes invalides (pointe vers lui mme)");
						}else
							statusLabel.setText("Coordonnes invalides");
					}catch(NumberFormatException e){
						statusLabel.setText("Coordonnes invalides");
					}
				}else
					statusLabel.setText("Veuillez entrer une combinaison activer le tlporteur");
			}			
		});
		
		this.getContentPane().add(conteneurPanel, BorderLayout.CENTER);
		this.getContentPane().add(boutonPanel, BorderLayout.SOUTH);
	}
}
