package org.system.newEditor;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import org.system.Carte;
import org.system.Teleporteur;

/**
 * Boite de dialogue qui affiche les erreurs de la carte :
 * pas de trsor, tlporteur qui envoir sur une mauvaise case...
 * @author Groupe 6 - 2009
 */
public class DialogVerifierCarte extends JDialog{

	private static final long serialVersionUID = 1L;
	
	private Carte carte;
	private JTextPane erreurTextPane;
	private JButton fermerBouton;
	
	public DialogVerifierCarte(JFrame parent, Carte carte){
		super(parent, "Vrification des erreurs sur la carte", true);
		
		this.setSize(400, 300);
		this.setLocationRelativeTo(null);
		this.setResizable(false);
		this.setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		
		this.carte = carte;
		
		this.initialiserComposant();
	}
	
	public void initialiserComposant(){
		
		erreurTextPane = new JTextPane();
		fermerBouton = new JButton("Fermer");
		
		erreurTextPane.setEditable(false);
		erreurTextPane.setOpaque(true);
		
		try {
			verifierCarte();
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
		
		JPanel boutonPanel = new JPanel();
		boutonPanel.add(fermerBouton);
		
		fermerBouton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				setVisible(false);
			}
		});
		
		this.getContentPane().add(erreurTextPane, BorderLayout.CENTER);
		this.getContentPane().add(boutonPanel, BorderLayout.SOUTH);
	}

	public void verifierCarte() throws BadLocationException{
		StyledDocument doc = erreurTextPane.getStyledDocument();
		addStylesToDocument(doc);

		doc.insertString(doc.getLength(), "Tlporteur :\n", doc.getStyle("bold"));
		for(int i = 0;i<carte.getLargeur();i++){
        	for(int j = 0;j<carte.getLongueur();j++){
        		if(carte.getCase(i,j).getClass().getName().equals("org.system.Teleporteur")){
        			if(carte.getCase(i,j).isFranchissable() == false)
        				doc.insertString(doc.getLength(), "  Le tlporteur n'est pas accessible.\n", doc.getStyle("regular"));
        		
        			/* Test si le tlporteur n'envoie pas sur une case non franchissable */
        			if(carte.getCase(((Teleporteur) carte.getCase(i,j)).getArriverX()/16, ((Teleporteur) carte.getCase(i,j)).getArriverY()/16).isFranchissable() == false)
        				doc.insertString(doc.getLength()," le tlporteur en "+i+","+j+" envoie sur une mauvaise case ("+(((Teleporteur) carte.getCase(i,j)).getArriverX()/16)+", "+(((Teleporteur) carte.getCase(i,j)).getArriverY()/16)+". \n", doc.getStyle("regular"));
        		}
        	}
		}

		/*
		 * Code pour vrifier que tous les lments obligatoires
		 * sur la carte sont l, et qu'ils sont accessibles
		 */
		boolean tresorDejaPose = false;
	boolean clefDejaPose = false;
	boolean incantationDejaPose = false;
	int nbIndice=0,indiceFranch;
	int nbEquipe = carte.getEquipeInfo().getNombreEquipe();
	
	for(int i=0; i<carte.getLargeur(); i++){
		for(int j=0; j<carte.getLongueur(); j++){
			if(carte.getCase(i, j).getClass().getName().equals("org.system.Tresor")){
				tresorDejaPose = true;
				if(tresorDejaPose==true){
					if(carte.getCase(i, j).isFranchissable() == false){
						doc.insertString(doc.getLength(), "  Le trsor n'est pas accessible.\n", doc.getStyle("regular"));
					}
					if(carte.tresor.isVisible() == true)
						doc.insertString(doc.getLength(), "  Le trsor est visible.\n", doc.getStyle("regular"));
				}
					
			}else if(carte.getCase(i, j).getClass().getName().equals("org.system.Clef")){
				clefDejaPose = true;
				if(clefDejaPose==true){
					if(carte.getCase(i, j).isFranchissable() == false){
						doc.insertString(doc.getLength(), "  La cl n'est pas accessible.\n", doc.getStyle("regular"));
					}
				}
				
			}else if(carte.getCase(i, j).getClass().getName().equals("org.system.Incantation")){
				incantationDejaPose = true;
				if(incantationDejaPose==true){
					if(carte.getCase(i, j).isFranchissable() == false){
						doc.insertString(doc.getLength(), "  L'incantation n'est pas accessible.\n", doc.getStyle("regular"));
					}
				}
				
			}
			
			else if(carte.getCase(i, j).getClass().getName().equals("org.system.Indice")){
				nbIndice++;
							}
		}
	}
	
	if(tresorDejaPose==false){
		doc.insertString(doc.getLength(), "  Pas de trsor.\n", doc.getStyle("regular"));
	}
	if(clefDejaPose==false){
		doc.insertString(doc.getLength(), "  Pas de cl.\n", doc.getStyle("regular"));
	}
	if(incantationDejaPose==false){
		doc.insertString(doc.getLength(), "  Pas d'incantation.\n", doc.getStyle("regular"));
	}
	
	//Concernant les indices
	if(nbIndice<nbEquipe){
		doc.insertString(doc.getLength(), "  Il n'y a pas d'indice au dpart de chaque quipe\n", doc.getStyle("regular"));
	}
	else{
		indiceFranch=nbIndice;
		for(int i=0; i<carte.getLargeur(); i++){
			for(int j=0; j<carte.getLongueur(); j++){
				if(carte.getCase(i, j).getClass().getName().equals("org.system.Indice")){
					if(carte.getCase(i, j).isFranchissable() == false){
						indiceFranch--;
					}
				}
				
			}
			
		}
		if(nbIndice-indiceFranch!=0){
			doc.insertString(doc.getLength(), "  Il y a "+(nbIndice-indiceFranch)+"indice(s) non accessible(s)\n", doc.getStyle("regular"));
		}
		else{
			for(int i=0; i<nbEquipe; i++){
				int x=(int)carte.getEquipeInfo().getDepart(i).getX();
				int y=(int)carte.getEquipeInfo().getDepart(i).getY();
				if(!(carte.getCase(x,y).getClass().getName().equals("org.system.Indice"))){
					doc.insertString(doc.getLength(), "  Pas d'indice pour l'quipe"+(i+1)+"  la case ["+x+"]["+y+"]\n", doc.getStyle("regular"));
				}
		}
			
		}
	}
	
	
	}
	
	
	
	
	
	private void addStylesToDocument(StyledDocument doc) {
        Style def = StyleContext.getDefaultStyleContext().
                        getStyle(StyleContext.DEFAULT_STYLE);

        Style regular = doc.addStyle("regular", def);

        Style s = doc.addStyle("italic", regular);
        StyleConstants.setItalic(s, true);

        s = doc.addStyle("bold", regular);
        StyleConstants.setBold(s, true);

        s = doc.addStyle("small", regular);
        StyleConstants.setFontSize(s, 10);

        s = doc.addStyle("large", regular);
        StyleConstants.setFontSize(s, 16);
    }
}
