package org.system.newEditor;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.system.Carte;
import org.system.Case;

/** Fenetre principale qui contient une carte, un afficheur de carte, une barre de menu et differents panneaux d'objets
 * 	implemente les evenements d'interaction de la barre de menu, et permet d'agir en consquence sur la carte ou l'afficheur */
public class FenetreEditeur extends JFrame implements WindowListener{

	private static final long serialVersionUID = 1L;
	
	/* LISTE MONTAGNE */
	private static ListeTexture listeTextureMontagneTerrain;
	private static ListeTexture listeTextureMontagneObstacle;
	private static ListeTexture listeTextureMontagneBatiment;
	
	/* LISTE TROPICAL */
	private static ListeTexture listeTextureTropicalTerrain;
	private static ListeTexture listeTextureTropicalObstacle;
	private static ListeTexture listeTextureTropicalBatiment;
	
	/* LISTE CHATEAU */
	private static ListeTexture listeTextureChateauTerrain;
	private static ListeTexture listeTextureChateauObstacle;
	
	/* LISTE PARTICULIERES */
	private static ListeTexture listeTextureBasique;
	
	/** l'interface graphique ou est affiche la carte */
	private AfficheurCarte afficheur;
	/** la carte dite */
	private Carte carte;
	private BarreMenu barreMenu;
	private BarreEtat barreEtat;
	private PanneauTexture panneauTexture;
	private PanneauPnj panneauPnj;
	private PanneauTransport panneauTransport;
	
	public FenetreEditeur(){
		
		this.setSize(600, 500);
		this.setLocationRelativeTo(null);
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.setIconImage(new ImageIcon("chasse_au_tresor.jpg").getImage());

		
		chargerTexture();
		
		carte = new Carte(20,20,null);
		carte.initDark();
		
		
		barreMenu = new BarreMenu();
		
		barreEtat = new BarreEtat();
		
		afficheur = new AfficheurCarte(carte, this);
		JPanel conteneurAfficheur = new JPanel();
		conteneurAfficheur.add(afficheur);
		
		remplirCarte("herbeHaute.png");
		
		panneauTexture = new PanneauTexture(this);
		panneauPnj = new PanneauPnj(this);
		panneauTransport = new PanneauTransport(this);
		
		this.setTitle("Chasse au trsor - Editeur de niveaux [" + carte.getNomCarte()+"]");
		
		this.setJMenuBar(barreMenu);
		
		this.getContentPane().add(conteneurAfficheur, BorderLayout.CENTER);
		this.getContentPane().add(barreEtat, BorderLayout.SOUTH);
		
		/* La fenetre principale "ecoute" les evenements clavier pour l'afficheur
		 * (l'afficheur utilisera ces infos) */
		this.addKeyListener(afficheur);
		this.addWindowListener(this);
		
		
		/* A partir d'ici, on dfini les actions quand on appuie sur un bouton de la barre de menu
		 * fonctionnement un peu identique pour toutes les actions */
		barreMenu.getActionNouveau().addActionListener(new ActionListener(){ //ajout d'ecouteur d'action
			public void actionPerformed(ActionEvent e) { //methode dclench par l'action
				//on ouvre la boite de dialogue "nouvelle carte"
				DialogNouveau dialogNouveau = new DialogNouveau(null);
				
				// la boite a t ferme (soit en cliquant sur "fermer", "annuler" ou "ok")
				if(dialogNouveau.getCarte() != null){ //si la carte cre n'est pas nulle (donc clic sur "ok")
					carte = dialogNouveau.getCarte(); //on recupere la carte
					//remplirCarte("herbeHaute.png");
					afficheur.setCarte(carte); //et on remplace l'actuelle par la nouvelle
					afficheur.moveTo(0,0);
					barreMenu.getActionActiverModeAucun().doClick();
					setTitle("Chasse au trsor - Editeur de niveaux [" + carte.getNomCarte()+"]");
					AfficheurCarte.getListeAnnuler().getListeCase().clear();
					AfficheurCarte.getListeRetablir().getListeCase().clear();
				}
			}
		});
		
		barreMenu.getActionQuitter().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				@SuppressWarnings("unused")
				DialogQuitter quit = new DialogQuitter(null, carte);
			}
		});
		
		barreMenu.getActionEnregistrer().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				@SuppressWarnings("unused")
				DialogEnregistrer dialogEnregistrer = new DialogEnregistrer(null, carte);
				setTitle("Chasse au trsor - Editeur de niveaux [" + carte.getNomCarte()+"]");
			}
		});
		
		barreMenu.getActionExporterImage().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				@SuppressWarnings("unused")
				DialogExporterImage dialogExporter = new DialogExporterImage(null, carte);
			}
		});
		
		barreMenu.getActionProprieteCarte().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				@SuppressWarnings("unused")
				DialogProprieteCarte dialog = new DialogProprieteCarte(null, carte);
			}
		});
		
		barreMenu.getActionOuvrir().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				DialogOuvrir dialogOuvrir = new DialogOuvrir(null);
				
				if(dialogOuvrir.getCarte() != null){
					AfficheurCarte.getListeAnnuler().getListeCase().clear();
					AfficheurCarte.getListeRetablir().getListeCase().clear();
					carte = dialogOuvrir.getCarte();
					//carte.getListeTransp().clear();
					afficheur.setCarte(carte);
					afficheur.moveTo(0,0);
					setTitle("Chasse au trsor - Editeur de niveaux [" + carte.getNomCarte()+"]");
				}
			}
		});
		
		barreMenu.getActionAnnuler().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				int caseX;
				int caseY;
				if(AfficheurCarte.getListeAnnuler().getListeCase().size() > 0){
					ArrayList<SauvegardeCase> tmpListe = new ArrayList<SauvegardeCase>();
					for(int i=0;i<AfficheurCarte.getListeAnnuler().derniereSauvegarde().size();i++){
						caseX = AfficheurCarte.getListeAnnuler().derniereSauvegarde().get(i).getX();
						caseY = AfficheurCarte.getListeAnnuler().derniereSauvegarde().get(i).getY();
						tmpListe.add(new SauvegardeCase(carte.getCase(caseX, caseY), caseX, caseY));
					}
					AfficheurCarte.getListeRetablir().ajouterSauvegarde(tmpListe);
					AfficheurCarte.getListeAnnuler().restaurer(carte);
					AfficheurCarte.getListeAnnuler().supprimerDerniereSauvegarde();
					afficheur.repaint();
				}
			}
		});
		
		barreMenu.getActionRetablir().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				int caseX;
				int caseY;
				if(AfficheurCarte.getListeRetablir().getListeCase().size() > 0){
					ArrayList<SauvegardeCase> tmpListe = new ArrayList<SauvegardeCase>();
					for(int i=0;i<AfficheurCarte.getListeRetablir().derniereSauvegarde().size();i++){
						caseX = AfficheurCarte.getListeRetablir().derniereSauvegarde().get(i).getX();
						caseY = AfficheurCarte.getListeRetablir().derniereSauvegarde().get(i).getY();
						tmpListe.add(new SauvegardeCase(carte.getCase(caseX, caseY), caseX, caseY));
					}
					AfficheurCarte.getListeAnnuler().ajouterSauvegarde(tmpListe);
					AfficheurCarte.getListeRetablir().restaurer(carte);
					AfficheurCarte.getListeRetablir().supprimerDerniereSauvegarde();
					afficheur.repaint();
				}
			}
		});
		
		barreMenu.getActionRemplirCarte().addActionListener(new ActionListener(){ //ajout d'ecouteur d'action
			public void actionPerformed(ActionEvent e) { //methode dclench par l'action
				//on ouvre la boite de dialogue "nouvelle carte"
				@SuppressWarnings("unused")
				DialogRemplirCarte dialogRemplirCarte = new DialogRemplirCarte(null,carte);
				
			}
		});
		
		barreMenu.getActionVerifierCarte().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				DialogVerifierCarte dialog = new DialogVerifierCarte(null, carte);
				dialog.setVisible(true);
			}
		});
		
		barreMenu.getActionPanneauTexture().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(barreMenu.getActionPanneauTexture().isSelected()){
					afficheur.setMode(AfficheurCarte.MODE_TEXTURE);
					panneauTexture.setVisible(true);
					panneauPnj.setVisible(false);
					afficheur.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
				}
				afficheur.repaint();
			}
		});
		
		barreMenu.getActionPanneauPnj().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(barreMenu.getActionPanneauPnj().isSelected()){
					afficheur.setMode(AfficheurCarte.MODE_PNJ);
					panneauPnj.setVisible(true);
					panneauTexture.setVisible(false);
					afficheur.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				}
				afficheur.repaint();
			}
		});
		
		barreMenu.getActionPanneauTransport().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(barreMenu.getActionPanneauTransport().isSelected()){
					afficheur.setMode(AfficheurCarte.MODE_TRANSPORT);
					panneauTransport.setVisible(true);
					panneauTexture.setVisible(false);
					panneauPnj.setVisible(false);
					afficheur.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				}
				afficheur.repaint();
			}
		});
		
		barreMenu.getActionActiverFranchissabilite().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(barreMenu.getActionActiverFranchissabilite().isSelected()){
					afficheur.setMode(AfficheurCarte.MODE_FRANCHISSABILITE);
					afficheur.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
					panneauTexture.setVisible(false);
					panneauPnj.setVisible(false);
				}
				afficheur.repaint();
			}
		});
		
		barreMenu.getActionActiverModeAucun().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(barreMenu.getActionActiverModeAucun().isSelected()){
					afficheur.setMode(AfficheurCarte.MODE_AUCUN);
					afficheur.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					panneauTexture.setVisible(false);
					panneauPnj.setVisible(false);
				}
				afficheur.repaint();
			}
		});
		
		barreMenu.getActionAfficherElement().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(barreMenu.getActionAfficherElement().isSelected()){
					afficheur.setMode(AfficheurCarte.MODE_ELEMENT);
					afficheur.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
					panneauTexture.setVisible(false);
					panneauPnj.setVisible(false);
				}
				afficheur.repaint();
			}
		});
		
		barreMenu.getActionAfficherGrille().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(barreMenu.getActionAfficherGrille().isSelected()){
					afficheur.setAfficherGrille(true);
				}else{
					afficheur.setAfficherGrille(false);
				}
				afficheur.repaint();
			}
		});
		
		barreMenu.getActionAfficherCurseur().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(barreMenu.getActionAfficherCurseur().isSelected()){
					afficheur.setAfficherCurseur(true);
				}else{
					afficheur.setAfficherCurseur(false);
				}
				afficheur.repaint();
			}
		});
		
		barreMenu.getActionAPropos().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				DialogAPropos dialog = new DialogAPropos(null);
				dialog.setVisible(true);
			}
		});
		
		barreMenu.getActionAide().addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				DialogAide dialog = new DialogAide(null);
				dialog.setVisible(true);
			}
		});
		
		
		panneauTexture.addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e){
            	barreMenu.getActionActiverModeAucun().setSelected(true);
            	afficheur.setMode(AfficheurCarte.MODE_AUCUN);
            }
		});
		
		panneauPnj.addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e){
            	barreMenu.getActionActiverModeAucun().setSelected(true);
            	afficheur.setMode(AfficheurCarte.MODE_AUCUN);
            }
		});
		
		this.setVisible(true);
	}
	/* fin  couteurs */
	
	/** charge toutes les textures du jeu */
	public void chargerTexture(){
		/* CHARGEMENT LISTE MONTAGNE */
		listeTextureMontagneTerrain = new ListeTexture();
		listeTextureMontagneObstacle = new ListeTexture();
		listeTextureMontagneBatiment = new ListeTexture();
		
		try{
			listeTextureMontagneTerrain.chargerDossier("Graphics/Texture/montagne/terrain");
			listeTextureMontagneObstacle.chargerDossier("Graphics/Texture/montagne/obstacle");
			listeTextureMontagneBatiment.chargerDossier("Graphics/Texture/montagne/batiment");
		}catch(NullPointerException e1){
			e1.printStackTrace();
		}
		
		/* GHARGEMENT TROPICAL */
		listeTextureTropicalTerrain = new ListeTexture();
		listeTextureTropicalObstacle = new ListeTexture();
		listeTextureTropicalBatiment = new ListeTexture();
		
		try{
			listeTextureTropicalTerrain.chargerDossier("Graphics/Texture/tropical/terrain");
			listeTextureTropicalObstacle.chargerDossier("Graphics/Texture/tropical/obstacle");
			listeTextureTropicalBatiment.chargerDossier("Graphics/Texture/tropical/batiment");
		}catch(NullPointerException e1){
			e1.printStackTrace();
		}
		
		/* CHARGEMENT CHATEAU */
		listeTextureChateauTerrain = new ListeTexture();
		listeTextureChateauObstacle = new ListeTexture();
		
		try{
			listeTextureChateauTerrain.chargerDossier("Graphics/Texture/chateau/terrain");
			listeTextureChateauObstacle.chargerDossier("Graphics/Texture/chateau/obstacle");
		}catch(NullPointerException e1){
			e1.printStackTrace();
		}
		
		/* CHARGEMENT PARTICULIER */
		
		listeTextureBasique = new ListeTexture();
		
		try{
			listeTextureBasique.ajouterTexture(listeTextureMontagneTerrain.trouveByNom("herbe.png"));
			listeTextureBasique.ajouterTexture(listeTextureMontagneTerrain.trouveByNom("herbeHaute.png"));
			listeTextureBasique.ajouterTexture(listeTextureMontagneTerrain.trouveByNom("mer1.png"));
			listeTextureBasique.ajouterTexture(listeTextureMontagneTerrain.trouveByNom("sable1.png"));
			listeTextureBasique.ajouterTexture(listeTextureMontagneTerrain.trouveByNom("sentier1.png"));
			
			listeTextureBasique.ajouterTexture(listeTextureTropicalTerrain.trouveByNom("herbe.png"));
			listeTextureBasique.ajouterTexture(listeTextureTropicalTerrain.trouveByNom("mer.png"));
			listeTextureBasique.ajouterTexture(listeTextureTropicalTerrain.trouveByNom("sable0.png"));
			
			listeTextureBasique.ajouterTexture(listeTextureChateauTerrain.trouveByNom("herbe1.png"));
			listeTextureBasique.ajouterTexture(listeTextureChateauTerrain.trouveByNom("plancher1.png"));
			listeTextureBasique.ajouterTexture(listeTextureChateauTerrain.trouveByNom("sol2.png"));
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/** rempli la carte de nouvelles cases de texture
	 * a mettre dans Carte.java */
	public void remplirCarte(String texture){
		int i, j;
		try {
			if(listeTextureMontagneTerrain.trouveByNom(texture) != null){
				for(i=0;i<carte.getLargeur();i++){
			    	for(j=0;j<carte.getLongueur();j++){
			    		this.carte.getNiveau()[i][j] = new Case(listeTextureMontagneTerrain.trouveByNom(texture), true, true, false);
			    		this.carte.getCase(i,j).setSourceImage(listeTextureMontagneTerrain.trouveByNom(texture).getSource());
			    	}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public AfficheurCarte getAfficheur(){
		return afficheur;
	}
	
	public static ListeTexture getListeTextureMontagneTerrain() {
		return listeTextureMontagneTerrain;
	}

	public static ListeTexture getListeTextureMontagneObstacle() {
		return listeTextureMontagneObstacle;
	}

	public static ListeTexture getListeTextureMontagneBatiment() {
		return listeTextureMontagneBatiment;
	}

	public static ListeTexture getListeTextureTropicalTerrain() {
		return listeTextureTropicalTerrain;
	}

	public static ListeTexture getListeTextureTropicalObstacle() {
		return listeTextureTropicalObstacle;
	}

	public static ListeTexture getListeTextureTropicalBatiment() {
		return listeTextureTropicalBatiment;
	}

	public static ListeTexture getListeTextureChateauTerrain() {
		return listeTextureChateauTerrain;
	}

	public static ListeTexture getListeTextureChateauObstacle() {
		return listeTextureChateauObstacle;
	}

	public void setBarreEtat(BarreEtat barreEtat) {
		this.barreEtat = barreEtat;
	}

	public BarreEtat getBarreEtat() {
		return barreEtat;
	}

	public static ListeTexture getListeTextureBasique() {
		return listeTextureBasique;
	}

	public void windowActivated(WindowEvent arg0) {
	}

	public void windowClosed(WindowEvent arg0) {
    }

	public void windowClosing(WindowEvent arg0) {
        @SuppressWarnings("unused")
		DialogQuitter quit = new DialogQuitter(null, carte);
    }

	public void windowDeactivated(WindowEvent arg0) {
	}

	public void windowDeiconified(WindowEvent arg0) {
	}

	public void windowIconified(WindowEvent arg0) {
	}

	public void windowOpened(WindowEvent arg0) {
	}
}