package org.system.newEditor;

import java.util.ArrayList;
import java.util.Stack;

import org.system.Carte;

/** Listes de SauvegardeCase pouvant tre restaure, translate...
 * (c'est une liste de liste de case) sert a Annuler, Restaurer, Selectionner...
 * possde une taille maximale TAILLEMAX 
 * @author Groupe 6 - 2009
 */
public class ListeSauvegardeCase {
	
	public static final int TAILLEMAX = 100;
	
	private Stack<ArrayList<SauvegardeCase>> listeCase;
	
	public ListeSauvegardeCase(){
		listeCase = new Stack<ArrayList<SauvegardeCase>>();
	}
	
	/** ajoute une case a la liste. Si la taille vaut TAILLEMAX,
	 * le dernier lment de la liste est supprim pour faire de la place */
	public void ajouterSauvegarde(SauvegardeCase sauvegarde){
		ArrayList<SauvegardeCase> tmpListe = new ArrayList<SauvegardeCase>();
		tmpListe.add(sauvegarde); 
		
		if(listeCase.size() < TAILLEMAX){
			listeCase.add(tmpListe);
		}else{
			listeCase.remove(0);
			listeCase.add(tmpListe);
		}
	}
	
	/** ajoute une liste de cases  la liste. Si la taille vaut TAILLEMAX,
	 * le dernier lment de la liste est supprim pour faire de la place */
	public void ajouterSauvegarde(ArrayList<SauvegardeCase> liste){
		if(listeCase.size() < TAILLEMAX){
			listeCase.add(liste);
		}else{
			listeCase.remove(0);
			listeCase.add(liste);
		}
	}
	
	/** Remplace les cases de la carte par celle de la liste */
	public void restaurer(Carte carte){
		if(listeCase.size() > 0){
			for(int j=0; j<listeCase.lastElement().size(); j++){
				carte.getNiveau()[listeCase.lastElement().get(j).getX()][listeCase.lastElement().get(j).getY()] = listeCase.lastElement().get(j).getCase();
			}
		}
	}
	
	public void supprimerDerniereSauvegarde(){
		if(listeCase.size() > 0){
			listeCase.remove(listeCase.size()-1);
		}
	}

	public Stack<ArrayList<SauvegardeCase>> getListeCase() {
		return listeCase;
	}
	
	public ArrayList<SauvegardeCase> derniereSauvegarde(){
		try{
			return listeCase.lastElement();
		}catch(Exception e1){
			return null;
		}
	}
	
	/**
	 * Translate une liste de SauvegardeCase si aucune d'entre elles ne dpassent les bords de la carte
	 * @param liste Liste de SauvegardeCase  translater
	 * @param carte Carte utilise pour connaitre sa taille
	 * @param vx Dplacement sur l'axe x
	 * @param vy Dplacement sur l'axe y
	 * @return true(vrai) si le dplacement a t effectu, et false(faux) sinon
	 */
	public static boolean translater(ArrayList<SauvegardeCase> liste, Carte carte, int vx, int vy){
		boolean possible = true;
		int i=0;
		while(i<liste.size() && possible == true){
			if((liste.get(i).getX()+vx) < 0 || (liste.get(i).getX()+vx) >= carte.getLongueur() || (liste.get(i).getY()+vy) < 0 || (liste.get(i).getY()+vy) >= carte.getLargeur()){
				possible = false;
			}
			i++;
		}
		
		/* On a vrifi que toutes les cases peuvent se translater sans probleme, alors on le fait */
		if(possible == true){
			for(i=0; i<liste.size(); i++){
				liste.get(i).setX(liste.get(i).getX() + vx);
				liste.get(i).setY(liste.get(i).getY() + vy);
			}
		}
		
		return possible;
	}
}
