package org.system.newEditor;

import java.io.File;
import java.util.ArrayList;

import org.system.Texture;

/**
 * Une liste de textures, avec des mthodes pour la manipuler
 * @author Groupe 6 - 2009
 */
public class ListeTexture {

	private ArrayList<Texture> liste;
	
	/** Cr une liste vide */
	public ListeTexture(){
		liste = new ArrayList<Texture>();
	}
	
	/**
	 * Cre une texture et l'ajoute a la liste si l'image correspondante existe,
	 * et si elle n'y est pas dj
	 * @param source Lien relatif vers l'image de la Texture
	 */
	public void ajouterTexture(String source) {
		if(this.contient(source) == false){
			try{
				Texture texture = new Texture(source);
				liste.add(texture);
			}catch(NullPointerException e1){}
		}
	}
	
	/**
	 * Ajoute une texture a la liste si elle n'y est pas dj
	 * @param texture
	 */
	public void ajouterTexture(Texture texture) {
		if(texture != null){
			if(this.contient(texture.getSource()) == false){
				liste.add(texture);
			}
		}
	}
	
	/**
	 * Compare le lien vers l'image de chacun des lments de la liste
	 *  celui pass en paramtre pour savoir si la texture est dans la liste
	 * @param source Lien vers l'image
	 * @return true(vrai) si la texture est dans la liste ou false(faux) dans le cas contraire
	 */
	public boolean contient(String source){
		int i;
		
		for(i=0;i<liste.size();i++){
			if(liste.get(i).getSource().equals(source)){
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Compare le lien vers l'image de chacun des lments de la liste
	 *  celui pass en paramtre pour savoir si la texture est dans la liste
	 * @param source Lien vers l'image
	 * @return La texture si elle est dans la liste, sinon null
	 */
	public Texture trouveBySource(String source){
		int i;
		
		for(i=0;i<liste.size();i++){
			if(liste.get(i).getSource().equals(source)){
				return liste.get(i);
			}
		}
		
		return null;
	}
	
	/**
	 * Compare le nom de l'image de chacun des lments de la liste
	 *  celui pass en paramtre pour savoir si la texture est dans la liste
	 * @param source Nom de l'image
	 * @return La texture si elle est dans la liste, sinon null
	 */
	public Texture trouveByNom(String nom) throws Exception{
		int i;
		
		for(i=0;i<liste.size();i++){
			if(liste.get(i).getNom().equals(nom)){
				return liste.get(i);
			}
		}
		
		throw new Exception("Texture non trouve");
	}
	
	/**
	 * Parcours un dossier et charge toute les images dans la liste
	 * @param dossier Nom du dossier
	 * @throws NullPointerException Si le dossier n'existe pas
	 */
	public void chargerDossier(String dossier) throws NullPointerException{
		File repertoire = new File(dossier);
		String [] listeFichier;
		int i;
		
		if(dossier.endsWith("/") == false)
			dossier += "/";
		
		if(repertoire.exists()){
			
			listeFichier = repertoire.list();
			
			for(i=0; i<listeFichier.length; i++){
				if(listeFichier[i].endsWith(".jpg")){
					ajouterTexture(dossier+listeFichier[i]);
				}else if(listeFichier[i].endsWith(".png")){
					ajouterTexture(dossier+listeFichier[i]);
				}else if(listeFichier[i].endsWith(".gif")){
					ajouterTexture(dossier+listeFichier[i]);
				}
			}
		}else{
			throw new NullPointerException("Dossier inexistant : " + dossier);
		}
	}
	
	public String toString(){
		return new String("ListeTexture[longueur:"+liste.size()+"]");
	}
	
	/**
	 * Renvoie un lment de la liste
	 * @param index Le numro de l'lment dans la liste
	 * @return la texture recherche
	 */
	public Texture getTexture(int index) {
		return liste.get(index);
	}
	
	public int getLongueur() {
		return liste.size();
	}

	public ArrayList<Texture> getListe() {
		return liste;
	}
}
