package org.system.newEditor;

import java.awt.Dimension;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

/**
 * Un driv de JPanel qui permet de recuperer une combinaison de touche
 * sous la forme d'un tableau de codes ascii.
 * @author Groupe 6 - 2009
 */
public class PanneauCombinaison extends JPanel{

	private static final long serialVersionUID = 1L;
	
	/** Liste des lettres pouvant composer la combinaison */
	private final String[] listeLettre = {" ", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z"};
	
	/** Nombre de lettres composant la combinaison */
	private int nbLettre;
	
	/** Un tableau de liste droulante, pour choisir les lettres de la combinaison
	 *  Une liste par lettre de combinaison */
	private JComboBox[] combiBox;
	
	/** Titre du panneau de combinaison */
	private JLabel label;
	
	/**
	 * Cre un PanneauCombinaison
	 * @param label titre du panneau
	 * @param nbLettre nombre de lettres de la combinaison
	 * @param largeur largeur en pixel du panneau
	 */
	public PanneauCombinaison(String label, int nbLettre, int largeur){
		this.setPreferredSize(new Dimension(largeur, 60));
		this.nbLettre = nbLettre;
		this.label = new JLabel(label);
		this.label.setPreferredSize(new Dimension(largeur-20, 20));
		combiBox = new JComboBox[nbLettre];
		
		this.add(this.label);
		
		/* Cration des listes droulantes et ajout au panneau actuel */
		for(int i=0;i<this.nbLettre;i++){
			combiBox[i] = new JComboBox(listeLettre);
			this.add(combiBox[i]);
		}
	}
	
	/**
	 * Charge les valeurs listes droulantes depuis une combinaison
	 * @param combinaison La combinaison a charger
	 */
	public void setCombinaison(int[] combinaison){
		if(combinaison != null){
			int j=0;
			for(int i=0; i<combinaison.length; i++){
				/* un code ascii - 64 est l'indice de cette lettre dans la liste des lettes droulante */
				if(combinaison[i] - 64 >= 0){
					combiBox[j].setSelectedItem(new String(""+(char)combinaison[i]));
					j++;
				}
			}
		}
	}
	
	/**
	 * Convertit les valeurs des listes droulantes en tableau de codes ascii.
	 * Les doublons sont supprims.
	 * @return le tableau de codes ascii (int)
	 */
	public int[] getCombinaison(){
		int longueurCombinaison = 0;
		int i, j;
		
		/* On elimine les lettres dupliques */
		for(i=0;i<nbLettre;i++){
			for(j=0;j<nbLettre;j++){
				if(i != j)
					if(combiBox[i].getSelectedItem().toString().charAt(0) == combiBox[j].getSelectedItem().toString().charAt(0))
						combiBox[j].setSelectedIndex(0);
			}
		}
		
		/* Recuperation du nombre de lettres de la combinaison */
		for(i=0;i<nbLettre;i++)
			if(combiBox[i].getSelectedItem().toString() != " ")
				longueurCombinaison++;
		
		if(longueurCombinaison == 0) return null;
		
		/* Recuperation lettre par lettre le code ascii pour le mettre dans le tableau de int */
		int[] combinaison = new int[longueurCombinaison];
		j=0;
		for(i=0;i<nbLettre;i++){
			if(combiBox[i].getSelectedItem().toString() != " "){
				combinaison[j] = (int)combiBox[i].getSelectedItem().toString().charAt(0);
				j++;
			}
		}
		
		return combinaison;
	}
}
