package org.system.newEditor;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;

/**
 * Dtecte la slection d'une zone rectangulaire de cases sur l'interface graphique
 * Ncessite la touche Alt enfonce
 * @author Groupe 6 - 2009
 */
public class SelectionneurMultiple implements MouseListener, MouseMotionListener{

	/** p1 : coordonnes de la case au premier clic. p2 : coord de la case lors du relachement du clic */
	private Point p1, p2;
	/** Liste des cases slectionnes */
	private ArrayList<SauvegardeCase> liste;
	/** l'interface graphique */
	private AfficheurCarte afficheur;
	/** Hauteur et largeur de la selection, (en nombre de cases) */
	private int ecartX, ecartY;
	/** dtermine si on a selectionn une zone ou non */
	private boolean hasSelection;
	
	/** Constructeur : initialise les attributs et ajoute des couteurs */
	public SelectionneurMultiple(AfficheurCarte afficheur){
		this.afficheur = afficheur;
		hasSelection = false;
		p1 = null;
		p2 = null;
		liste = new ArrayList<SauvegardeCase>();
		afficheur.addMouseListener(this);
		afficheur.addMouseMotionListener(this);
	}
	
	/** Annule toute slection */
	public void reset(){
		p1 = null;
		p2 = null;
		liste = new ArrayList<SauvegardeCase>();
		hasSelection = false;
		afficheur.repaint();
	}
	
	/** Afficher le cadre de selection
	 * 4 mouvements sont possibles donc 4 affichages en consquence
	 */
	public void afficherSelection(Graphics g){
		if(p1 != null){
			/* On a deja une slection valide, on l'affiche en bleu.
			 * On affiche galement la liste de case a  */
			if(hasSelection == true){
				g.setColor(Color.BLUE);
				for(int i=0;i<liste.size();i++){
					g.drawImage(liste.get(i).getCase().getImageIcon().getImage(), liste.get(i).getX()*16, liste.get(i).getY()*16,null);
				}
			}
			else
				g.setColor(Color.RED);
			
			if(p1.x <= p2.x && p1.y <= p2.y){
				g.drawLine(p1.x, p1.y, p2.x+15, p1.y);
				g.drawLine(p1.x, p1.y, p1.x, p2.y+15);
				g.drawLine(p1.x, p2.y+15, p2.x+15, p2.y+15);
				g.drawLine(p2.x+15, p1.y, p2.x+15, p2.y+15);
			}else if(p1.x >= p2.x && p1.y >= p2.y){
				g.drawLine(p2.x, p2.y, p1.x+15, p2.y);
				g.drawLine(p2.x, p2.y, p2.x, p1.y+15);
				g.drawLine(p2.x, p1.y+15, p1.x+15, p1.y+15);
				g.drawLine(p1.x+15, p2.y, p1.x+15, p1.y+15);
			}else if(p1.x <= p2.x && p1.y >= p2.y){
				g.drawLine(p1.x, p2.y, p2.x+15, p2.y);
				g.drawLine(p1.x, p2.y, p1.x, p1.y+15);
				g.drawLine(p1.x, p1.y+15, p2.x+15, p1.y+15);
				g.drawLine(p2.x+15, p2.y, p2.x+15, p1.y+15);
			}else if(p1.x >= p2.x && p1.y <= p2.y){
				g.drawLine(p2.x, p1.y, p1.x+15, p1.y);
				g.drawLine(p2.x, p1.y, p2.x, p2.y+15);
				g.drawLine(p2.x, p2.y+15, p1.x+15, p2.y+15);
				g.drawLine(p1.x+15, p1.y, p1.x+15, p2.y+15);
			}
		}
	}
	
	/** Dtermine si le curseur est correctement situ sur l'afficheur */
	public boolean clicValide(MouseEvent e){
		if(e.getX() >= 0 && e.getY() >= 0 && e.getX()< afficheur.getWidth() && e.getY()< afficheur.getHeight())
			return true;
		else
			return false;
	}
	
	/** Un clic sur l'interface : nouvelle slection? */
	public void mousePressed(MouseEvent e) {
		if(hasSelection == true){
			afficheur.getFenetre().getBarreEtat().resetAide();
			reset();
		}
	}
	
	/** clic relach : slection finie */
	public void mouseReleased(MouseEvent e) {
		if(p1 != null){
			/* ce sont des points pour des calculs intermdiaires */
			Point pMin = new Point();
			Point pMax = new Point();
			
			/* Calcul qui fait p1 devenir le point le plus rapproch de l'origine,
			 * et p2 le plus loign */
			pMin.x = (Math.min(p1.x, p2.x)/16)*16;
			pMin.y = (Math.min(p1.y, p2.y)/16)*16;
			pMax.x = ((Math.max(p1.x, p2.x)/16))*16;
			pMax.y = ((Math.max(p1.y, p2.y)/16))*16;
			
			p1 = pMin;
			p2 = pMax;
			
			/* On peut maintenant calculer le nombre de cases entre p1 et p2 dans le sens horizontal et vertical */
			ecartX = (p2.x/16) - (p1.x/16);
			ecartY = (p2.y/16) - (p1.y/16);
			
			/* Copie des cases selectionnes dans une liste */
			for(int i=p1.x/16;i<=(p1.x/16)+ecartX;i++){
				for(int j=p1.y/16;j<=(p1.y/16)+ecartY;j++){
					liste.add(new SauvegardeCase(afficheur.getCarte().getCase(i,j), i, j));
				}
			}
			
			/* On a une slection ! */
			hasSelection = true;
			
			afficheur.getFenetre().getBarreEtat().setAide("Dplacez la slection avec les touches directionnelles et posez la avec la touche Entre");
			
			afficheur.repaint();
		}
	}
	
	public void mouseDragged(MouseEvent e) {
		/* on met a jour la case clique (si c'est le premier clic on cre les 2 points) */
		if(e.isAltDown()){
			 if(p2 != null){
				if(clicValide(e)){
					p2.x = (e.getX()/16)*16;
					p2.y = (e.getY()/16)*16;
				}
			}else if(p1 == null){	
				p1 = new Point((e.getX()/16)*16, (e.getY()/16)*16);
				p2 = new Point((e.getX()/16)*16, (e.getY()/16)*16);
			}
		}
	}
	
	
	
	public void mouseClicked(MouseEvent e) {reset();}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	
	public void mouseMoved(MouseEvent e) {}

	public void setP1(Point p1) {
		this.p1 = p1;
	}

	public Point getP1() {
		return p1;
	}

	public void setP2(Point p2) {
		this.p2 = p2;
	}

	public Point getP2() {
		return p2;
	}

	public ArrayList<SauvegardeCase> getListe() {
		return liste;
	}

	public int getEcartX() {
		return ecartX;
	}
	
	public int getEcartY() {
		return ecartY;
	}

	public boolean hasSelection() {
		return hasSelection;
	}
}
